#! /usr/bin/perl -w
################################################################################
# Copyright 2004-2011 MERETHIS
# Centreon is developped by : Julien Mathis and Romain Le Merlus under
# GPL Licence 2.0.
# 
# This program is free software; you can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by the Free Software 
# Foundation ; either version 2 of the License.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with 
# this program; if not, see <http://www.gnu.org/licenses>.
# 
# Linking this program statically or dynamically with other modules is making a 
# combined work based on this program. Thus, the terms and conditions of the GNU 
# General Public License cover the whole combination.
# 
# As a special exception, the copyright holders of this program give MERETHIS 
# permission to link this program with independent modules to produce an executable, 
# regardless of the license terms of these independent modules, and to copy and 
# distribute the resulting executable under terms of MERETHIS choice, provided that 
# MERETHIS also meet, for each linked independent module, the terms  and conditions 
# of the license of that module. An independent module is a module which is not 
# derived from this program. If you modify this program, you may extend this 
# exception to your version of the program, but you are not obliged to do so. If you
# do not wish to do so, delete this exception statement from your version.
# 
# For more information : contact@centreon.com
# 
# SVN : $URL: http://svn.centreon.com/trunk/plugins-2.x/src/check_centreon_snmp_multiple_process $
# SVN : $Id: check_centreon_snmp_multiple_process 10069 2010-02-24 11:05:19Z jmathis $
#
####################################################################################
#
# Script init
#

use strict;
use FindBin;
use lib "$FindBin::Bin";
use lib "@DIR_PLUGINS@";
use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
use Getopt::Long;
use vars qw($opt_h $opt_H $opt_p $opt_s $opt_t);

# Plugin var init

my($proc, $proc_run);

$PROGNAME = $0;
sub print_help ();
sub print_usage ();


Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "s=s"   => \$opt_s, "size=s" => \$opt_s,
     "p=s"   => \$opt_p, "port=s" => \$opt_p,
     "t=s"	=> \$opt_t, "type=s" => \$opt_t,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H);

if ($opt_h) {
  	print_help();
 	exit $ERRORS{'OK'};
}

my $nrpe_plugin = "@DIR_PLUGINS@/check_nrpe";
my $port = "5666";
my $size = "500";
my $type = "process";

if ($opt_p) {
	$port = $opt_p;
}

if ($opt_t) {
	$type = $opt_t;	
}

if ($opt_s) {
	$size = $opt_s;	
}

if ($type ne "process" && $type ne "services") {
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

if (!$opt_H) {
	print_usage();
	exit $ERRORS{'OK'};
}

my $name = $0;
my $max = 0;
my $discoveryNbPage;
my $discoveryCmd;

if ($type eq "process") {
	$discoveryNbPage = "discovery_process_number";
	$discoveryCmd = "discovery_process";
} elsif ($type eq "services") {
	$discoveryNbPage = "discovery_services_number";
	$discoveryCmd = "discovery_services";
}

my $cmd = `$nrpe_plugin -H $opt_H -p $port -c $discoveryNbPage -a $size`;
if ($cmd =~ /<data><total_number>(\d+)<\/total_number><\/data>/) {
	$max = ${1};
} else {
	exit $ERRORS{'UNKNOWN'};
}

my $part;
binmode(STDOUT, ":utf8");
print "<data>";
for ($part = 0; $part <= $max; $part++) {
	my $cmd2 = `$nrpe_plugin -H $opt_H -p $port -c $discoveryCmd -a $size $part`;
	$cmd2 =~ s/<data>//g;
	$cmd2 =~ s/<\/data>//g;
	print $cmd2;
}
print "</data>";



sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -t (--type)  Scan type, must be process or services (default to process)\n";
	print "   -s (--size)  Page size (default to 1000)\n";	
    print "   -p (--port)  NRPE Port (default to 5666)\n";
    print "   -h (--help)       usage help\n";
}
sub print_help () {
	print "##############################################\n";
    print "#    Copyright (c) 2004-2011 Centreon        #\n";
    print "#    Bugs to http://forge.centreon.com/      #\n";
    print "##############################################\n";
    print_usage();
    print "\n";
}
