#!/usr/bin/perl -w
##################################################################################
# Copyright 2005-2011 MERETHIS
# Centreon is developped by : Julien Mathis and Romain Le Merlus under
# GPL Licence 2.0.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation ; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses>.
#
# Linking this program statically or dynamically with other modules is making a
# combined work based on this program. Thus, the terms and conditions of the GNU
# General Public License cover the whole combination.
#
# As a special exception, the copyright holders of this program give MERETHIS
# permission to link this program with independent modules to produce an executable,
# regardless of the license terms of these independent modules, and to copy and
# distribute the resulting executable under terms of MERETHIS choice, provided that
# MERETHIS also meet, for each linked independent module, the terms  and conditions
# of the license of that module. An independent module is a module which is not
# derived from this program. If you modify this program, you may extend this
# exception to your version of the program, but you are not obliged to do so. If you
# do not wish to do so, delete this exception statement from your version.
#
# For more information : contact@centreon.com
#
# SVN : $URL :$
# SVN : $Id :$
#
####################################################################################

use strict;
use Net::SNMP qw(:snmp oid_lex_sort);
use FindBin;
use lib "$FindBin::Bin";
use lib "/usr/lib/nagios/plugins-nrpe";
use utils qw($TIMEOUT %ERRORS &print_revision &support);
if (eval "require centreon" ) {
    use centreon qw(get_parameters);
    use vars qw(%centreon);
    %centreon=get_parameters();
} else {
	print "Unable to load centreon perl module\n";
    exit $ERRORS{'UNKNOWN'};
}
use Getopt::Long;
use vars qw($PROGNAME $VERSION);

sub print_help ();
sub print_usage ();
sub snmp_connection ();
sub list_attributes();

#######################
# Defined Global Vars #
#######################

$PROGNAME = "$0";
$VERSION = "1.0";

my %OPTION = (
	"help" => undef, "version" => undef, "host" => undef, "process-show" => undef, "xml-attributes" => undef,
	"snmp-community" => "public", "snmp-version" => 1, "snmp-port" => 161, "snmp-auth-key" => undef, "snmp-auth-user" => undef, "snmp-auth-passwd" => undef,
	);

#############################
# Control command line args #
#############################
	
Getopt::Long::Configure('bundling');
GetOptions(
	"h|help"				=> \$OPTION{'help'},
    "V|version"				=> \$OPTION{'version'},
	"H|host=s"				=> \$OPTION{'host'},
	"w|warning=i"			=> \$OPTION{'warning'},
	"c|critical=i"			=> \$OPTION{'critical'},
	"C|community=s"			=> \$OPTION{'snmp-community'},
	"v|snmp-version=s"		=> \$OPTION{'snmp-version'},
	"P|snmp-port=i"			=> \$OPTION{'snmp-port'},
	"k|key=s"				=> \$OPTION{'snmp-auth-key'},
	"u|username=s"			=> \$OPTION{'snmp-auth-user'},
	"p|password=s"			=> \$OPTION{'snmp-auth-passwd'},
	"s|show"				=> \$OPTION{'interface-show'},
	"a|attributes"			=> \$OPTION{'xml-attributes'}
	
);

##################
# Verify Options #
##################

if (defined($OPTION{'version'})) {
    print_revision($PROGNAME, $VERSION);
  	exit $ERRORS{'OK'};
}

if (defined($OPTION{'help'})) {
    print_help();
    exit $ERRORS{'OK'};
}

if (defined($OPTION{'xml-attributes'})) {
	list_attributes();
	exit $ERRORS{'OK'};
}

if (!defined($OPTION{'interface-show'})) {
	print "Option -s (--show) is required";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

if (!defined($OPTION{'host'})) {
	print "Option -H (--host) is required";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

if (defined($OPTION{'snmp-version'}) && $OPTION{'snmp-version'} =~ /^([1-3])c?$/) {
	$OPTION{'snmp-version'} = $1;
} elsif (defined($OPTION{'snmp-version'})) {
	print "Unknown snmp version\n";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

if ($OPTION{'snmp-version'} eq "3") {
	if (!defined($OPTION{'snmp-auth-user'})) {
		print "Option -u (--username) is required for snmpV3\n";
		exit $ERRORS{'UNKNOWN'};
	}
	if (!defined($OPTION{'snmp-auth-passwd'}) && !defined($OPTION{'snmp-auth-key'})) {
		print "Option -k (--key) or -p (--password) is required for snmp v3\n";
		exit $ERRORS{'UNKNOWN'};
	} elsif (defined($OPTION{'snmp-auth-passwd'}) && defined($OPTION{'snmp-auth-key'})) {
		print "Only option -k (--key) or -p (--password) is needed for snmp v3\n";
		exit $ERRORS{'UNKNOWN'};
	}
}

if (defined($OPTION{'64-bits'}) && $OPTION{'snmp-version'} !~ /2/) {
	print "Error : Usage : SNMP v2 is required with option --64-bits\n";
	exit $ERRORS{'UNKNOWN'};
}

if (defined($OPTION{'64-bits'})) {
	if (eval "require bigint") {
	use bigint;
	} else {
		print "ERROR : Need bigint module for 64 bit counters\n"; 
		exit $ERRORS{"UNKNOWN"}
	}
}

##############
# Get values #
##############

my $OID_SW_RunName = $centreon{MIB2}{SW_RUNNAME};
my $OID_SW_RunParameters = $centreon{MIB2}{SW_RUNPARAMETERS};
my $OID_SW_RunIndex =$centreon{MIB2}{SW_RUNINDEX};
my $OID_SW_RunStatus =$centreon{MIB2}{SW_RUNSTATUS};

# Create SNMP connection
my ($result, $snmp_session, $error);
snmp_connection();

$result = $snmp_session->get_table(Baseoid => $OID_SW_RunName);
if (!defined($result)) {
    printf("UNKNOWN: %s.\n", $snmp_session->error);
    $snmp_session->close;
    exit $ERRORS{'UNKNOWN'};
}

my $processlistcount;
foreach my $key (oid_lex_sort(keys %$result)) {
	my $currentProcName = $result->{$key};
	if (!defined($processlistcount->{$currentProcName})) {
		$processlistcount->{$currentProcName} = 1;
	} else {
		$processlistcount->{$currentProcName}++;
	}
}

print "<data>\n";
foreach my $processName (%$processlistcount) {
	if (defined($processName) && defined($processlistcount->{$processName})) {
		print "<label name=\"" . $processName . "\" count=\"". $processlistcount->{$processName} ."\"/>\n";	
	}
}
print "</data>\n";

exit $ERRORS{'OK'};

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -C (--community)  SNMP read community (defaults to public)\n";
    print "                     used with SNMP v1 and v2c\n";
    print "   -v (--snmp-version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "                        3 for SNMP v3\n";
    print "   -P (--snmp-port)  SNMP port (default: 161)\n";
	print "   -k (--key)        snmp V3 key\n";
	print "   -u (--username)   snmp v3 username \n";
	print "   -p (--password)   snmp V3 password\n";
	print "\n";
	print "Disco options:\n";
	print "   -s (--show)       Show available process\n";
	print "   -a (--attributes) Show XML attributes\n";
	print "\n";
	print "Other options:\n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";

}

sub print_help () {
    print "##############################################\n";
    print "#    Copyright (c) 2004-2011 Centreon        #\n";
    print "#    Bugs to http://forge.centreon.com       #\n";
    print "##############################################\n";
    print_usage();
    print "\n";
}

sub trim($) {
        my $string = shift;
        $string =~ s/^\s+//;
        $string =~ s/\s+$//;
        return $string;
}

sub snmp_connection() {
	if ($OPTION{'snmp-version'} =~ /[1-2]/) {
		($snmp_session, $error) = Net::SNMP->session(-hostname 	=> $OPTION{'host'}, 
													-community 	=> $OPTION{'snmp-community'}, 
													-version 	=> $OPTION{'snmp-version'}, 
													-port 		=> $OPTION{'snmp-port'});
		if (!defined($snmp_session)) {
			print("UNKNOWN: SNMP Session : $error\n");
			exit $ERRORS{'UNKNOWN'};
		}
	} elsif (defined($OPTION{'snmp-auth-key'})) {
		($snmp_session, $error) = Net::SNMP->session(-hostname 	=> $OPTION{'host'}, 
													-version 	=> $OPTION{'snmp-version'}, 
													-username 	=> $OPTION{'snmp-auth-user'}, 
													-authkey 	=> $OPTION{'snmp-auth-key'}, 
													-port		=> $OPTION{'snmp-port'});
		if (!defined($snmp_session)) {
			print("UNKNOWN: SNMP Session : $error\n");
			exit $ERRORS{'UNKNOWN'};
		}
	} elsif (defined($OPTION{'snmp-auth-passwd'})) {
		($snmp_session, $error) = Net::SNMP->session(-hostname 	=> $OPTION{'host'}, 
													-version 	=> $OPTION{'snmp-version'}, 
													-username 	=> $OPTION{'snmp-auth-user'}, 
													-authpassword => $OPTION{'snmp-auth-passwd'}, 
													-port 		=> $OPTION{'snmp-port'});
		if (!defined($snmp_session)) {
			print("UNKNOWN: SNMP Session : $error\n");
			exit $ERRORS{'UNKNOWN'};
		}
	}
	
	if (defined($snmp_session)) {
		$snmp_session->translate(Net::SNMP->TRANSLATE_NONE);
	}
}

sub list_attributes() {
	print "<data>";
	print "<element>name</element>";
	print "<element>count</element>";
	print "</data>\n";
}
